package no.ntnu.jf.controller;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.Course;
import no.ntnu.jf.model.School;
import no.ntnu.jf.model.User;
import no.ntnu.jf.service.AdminService;
import no.ntnu.jf.service.CourseService;
import no.ntnu.jf.service.SchoolService;
import no.ntnu.jf.service.SpringLoginService;
import no.ntnu.jf.service.UserService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

@Controller
public class CourseController {

	@Autowired
	private CourseService courseService;
	
	@Autowired
	private AdminService adminService;
	
	@Autowired
	private SchoolService schoolService;
	
	@Autowired
	private UserService userService;
	
	private SpringLoginService loginService = new SpringLoginService();
	
	/*
	 *  Spring MVC Methods
	 */
	
	/***
	 * @author Jon Freberg
	 * @return A ModelAndView object that holds a list of courses
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/navigation/course", method = RequestMethod.GET)
	public ModelAndView listCourses() {
		ModelAndView mv = new ModelAndView();
		UserDetails adm = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(adm.getUsername());
		
		mv.setViewName("courses");
		mv.addObject("courselist", courseService.findAllCoursesForAdmin(admin.getId()));
		return mv;
	}
	
	/**
	 * @author Jon Freberg
	 * @param request
	 * @return A ModelAndView object that holds an admin object, a school object, a list of users and a list of courses
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/navigation/addUsersToCourse", method = RequestMethod.GET)
	public ModelAndView addUsersToCourse(HttpServletRequest request) {
		ModelAndView mv = new ModelAndView();
		mv.setViewName("addUsersToCourse");
		UserDetails adm = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(adm.getUsername());
		School school = schoolService.getSchoolByShortName(admin.getSchool().getSchoolShortName());
		List<User> userList = userService.findAllUsersAtSchool(admin.getSchool().getSchoolID());
		mv.addObject("admin", admin);
		mv.addObject("school", school);
		mv.addObject("userList", userList);
		mv.addObject("courseList", courseService.findAllCoursesForAdmin(admin.getId()));
		return mv;
	}
	
	/***
	 * @author Jon Freberg
	 * @param request
	 * @return A ModelAndView object that holds an admin object, and a school object
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/navigation/addOrEditCourse", method = RequestMethod.GET)
	public ModelAndView addOrEditCourse(HttpServletRequest request) {
		ModelAndView mv = new ModelAndView();
		mv.setViewName("addOrEditCourse");
		
		UserDetails user = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(user.getUsername());
		School school = schoolService.getSchoolByShortName(admin.getSchool().getSchoolShortName());
		
		mv.addObject("admin", admin);
		mv.addObject("school", school);

		
		return mv;
	}

	
	/*
	 *  CRUD Web Services
	 */
	
	/***
	 * @author Jon Freberg
	 * @param course (Format: as json object in the HTTP Body)
	 * @return Returns a valid course object if creation was valid, course object without ID if not.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/course", method = RequestMethod.POST)
	@ResponseBody
	public Course saveCourse(@RequestBody Course course) {
		courseService.saveCourse(course);
		return course;
	}
	
	/***
	 * @author Jon Freberg
	 * @param courseId
	 * @return Returns the course object that corresponds to the ID. Null if it does not exist.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/course/{courseId}", method = RequestMethod.GET)
	@ResponseBody
	public Course getCourseById(@PathVariable long courseId) {
		Course course = courseService.findCourseById(courseId);
		return course;
	}
	
	/***
	 * @author Jon Freberg
	 * @param userIds
	 * @param courseId
	 * @return Returns the course object the users are added to. Null if something went wrong.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/addUsersToCourse/{userIds}/{courseId}", method = RequestMethod.POST)
	@ResponseBody
	public Course addUsersToCourse(@PathVariable int[]userIds, @PathVariable long courseId) {
		Course course = courseService.addUsersToCourse(userIds, courseId);
		return course;
	}
	
	/***
	 * @author Jon Freberg
	 * @param course (Format: as json object in the HTTP Body)
	 * @return The course that got deleted. Null if something went wrong.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/course", method = RequestMethod.DELETE)
	@ResponseBody
	public Course deleteCourse(@RequestBody Course course) {
		courseService.removeCourse(course);
		return course;
	}
	
	/***
	 * @author Jon Freberg
	 * @param courseId
	 * @return A list of all the users that are currently not attending the course that corresponds to the ID passed as an argument.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/getUsersAvailableForCourse/{courseId}", method = RequestMethod.GET)
	@ResponseBody
	public List<User> getUsersAvailableForCourse(@PathVariable long courseId) {
		School school = courseService.findSchoolForCourse(courseId);
		List<User> usersNotInCourse = courseService.findAllUsersNotInCourseForSchool(courseId, school.getSchoolID());
		return usersNotInCourse;
	}
	
	/***
	 * @author Jon Freberg
	 * @param courseId
	 * @return A list of all users that are currently attending the course that corresponds to the ID passed as an argument.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/getUsersInCourse/{courseId}", method = RequestMethod.GET)
	@ResponseBody
	public List<User> getUsersInCourse(@PathVariable long courseId) {
		School school = courseService.findSchoolForCourse(courseId);
		List<User> usersInCourse = courseService.findAllUsersInCourseForSchool(courseId, school.getSchoolID());
		return usersInCourse;
	}

}
