package no.ntnu.jf.controller;

import javax.servlet.http.HttpServletRequest;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.School;
import no.ntnu.jf.model.User;
import no.ntnu.jf.service.AdminService;
import no.ntnu.jf.service.SchoolService;
import no.ntnu.jf.service.SpringLoginService;
import no.ntnu.jf.service.UserService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

@Controller
public class UserController {

	@Autowired
	private UserService userService;
	
	@Autowired
	private AdminService adminService;
	
	@Autowired
	private SchoolService schoolService;
	
	private SpringLoginService loginService = new SpringLoginService();
	
	/*
	 *  Spring MVC Methods
	 */
	
	/***
	 * @author Jon Freberg
	 * @param request
	 * @return A ModelAndView object that holds a list of users and a school object
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value="/navigation/users")
	public ModelAndView listUsers(HttpServletRequest request) {
		ModelAndView mv = new ModelAndView();
		mv.setViewName("users");
		
		UserDetails adm = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(adm.getUsername());
		School school = schoolService.getSchoolByShortName(admin.getSchool().getSchoolShortName());
		
		mv.addObject("userlist", userService.findAllUsersAtSchool(school.getSchoolID()));
		mv.addObject("school", school);
		
		return mv;
	}
	
	/*
	 * CRUD Web Services
	 */
	
	/***
	 * @author Jon Freberg
	 * @param user (Format: as json object in the HTTP Body)
	 * @return Returns a valid user object if creation was valid, user object without ID if not.
	 * 
	 * This method is only called from the iPad application when someone is registering a new user.
	 */
	@RequestMapping(value = "/fromipad/saveuser", method = RequestMethod.POST)
	@ResponseBody
	public User saveUserFromIpad(@RequestBody User user) {
		userService.addUser(user);
		return user;
	}
	
	/***
	 * @author Jon Freberg
	 * @param user (Format: as json object in the HTTP Body)
	 * @return Returns a valid user object if creation was valid, user object without ID if not.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/user", method = RequestMethod.POST)
	@ResponseBody
	public User saveUser(@RequestBody User user) {
		userService.addUser(user);
		return user;
	}
	
	/***
	 * @author Jon Freberg
	 * @param email
	 * @return The user object with the corresponding email. Null if user does not exist.
	 * 
	 * Regular expression added to email pathvariable to hinder Spring to cut of the email domain (.no/com)
	 */
	@RequestMapping(value="/user/{email:.+}", method = RequestMethod.GET)
	@ResponseBody
	public User getUser(@PathVariable String email) {
		User u = (User)userService.getUserByEmail(email);
		if(u == null) {
			System.out.println("User er NULL");
			return null;
		} else {
			return u;
		}  
	}
	
	/***
	 * @author Jon Freberg
	 * @param id
	 * @return The user object with the corresponding id. Null if user does not exist.
	 */
	@RequestMapping(value = "/user/{id}", method = RequestMethod.GET)
	@ResponseBody
	public User getUserById(@PathVariable String id) {
		long userId = Long.parseLong(id);
		User u = userService.getUserById(userId);	
		return u;
	}
	
	/***
	 * @author Jon Freberg
	 * @param email
	 * @param password
	 * @return A user object used to login to the iPad application.
	 */
	@RequestMapping(value = "/trylogin/{email:.+}/{password}", method = RequestMethod.GET)
	@ResponseBody
	public User tryLoginWithCredentials(@PathVariable String email, @PathVariable String password) {
		User temp = userService.tryLoginWithCreds(email, password);
		System.out.println("TRY LOGIN ER KALT P NYTT: EMAIL: " + email + " - PASSWORD: " + password);
		return temp;
	}
	
	
	/***
	 * @author Jon Freberg
	 * @param user
	 * @return A user object used to login to the iPad application.
	 */
	@RequestMapping(value = "/trylogin", method = RequestMethod.GET)
	@ResponseBody
	public User tryLogin(@RequestBody User user) {
		User temp = userService.tryLogin(user);
		return temp;
	}
	
}
